from .download import download_zip, download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "img.zip",down_path)
download_zip(xz_url + "MeiU_png.zip",down_path)

import bpy
import re
import os
import bpy.utils.previews
import webbrowser
from .translation import translate


bl_info = {
    "name": "Lazy Animation",
    "blender": (3, 4, 0),
    "category": "Object",
    "version": (1, 0, 0),
    "author": "MeiU",
    "description": "Animate selected objects along a custom curve path to the cursor position over a specified number of frames. Supports sequential and simultaneous keyframe modes, as well as reverse animation.",
    "location": "View3D > N > Lazy Animation",
    "warning": "",  
    "support": "COMMUNITY",
}

# 用于存储物体初始状态的字典
initial_state = {}

# 加载缩略图
preview_collections = {}
meiu_icon_collections = None

def load_previews():
    pcoll = bpy.utils.previews.new()

    icons_dir = os.path.join(os.path.dirname(__file__), "img")

    # 加载曲线类型图片
    pcoll.load("BEZIER", os.path.join(icons_dir, "bezier.png"), 'IMAGE')
    pcoll.load("PATH", os.path.join(icons_dir, "path.png"), 'IMAGE')

    # 加载关键帧模式图片
    pcoll.load("SEQUENTIAL", os.path.join(icons_dir, "Sequential.png"), 'IMAGE')
    pcoll.load("SIMULTANEOUS", os.path.join(icons_dir, "Simultaneous.png"), 'IMAGE')

    preview_collections["main"] = pcoll

def load_meiu_icons():
    global meiu_icon_collections
    meiu_icon_collections = bpy.utils.previews.new()

    icons_dir = os.path.join(os.path.dirname(__file__), "MeiU_png")

    # 加载 MeiU Info 面板的图标
    meiu_icon_collections.load("meiu_icon", os.path.join(icons_dir, "meiu.png"), 'IMAGE')
    meiu_icon_collections.load("discord_icon", os.path.join(icons_dir, "discord.png"), 'IMAGE')
    meiu_icon_collections.load("twitterx_icon", os.path.join(icons_dir, "twitterx.png"), 'IMAGE')
    meiu_icon_collections.load("Website_icon", os.path.join(icons_dir, "Website.png"), 'IMAGE')

def clear_previews():
    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections.clear()

    if meiu_icon_collections:
        bpy.utils.previews.remove(meiu_icon_collections)

def get_curve_items(self, context):
    pcoll = preview_collections["main"]
    return [
        ('BEZIER', "Bezier Curve", "", pcoll["BEZIER"].icon_id, 0),
        ('PATH', "Path", "", pcoll["PATH"].icon_id, 1),
    ]

def get_keymode_items(self, context):
    pcoll = preview_collections["main"]
    return [
        ('SEQUENTIAL', "Sequential", "", pcoll["SEQUENTIAL"].icon_id, 0),
        ('SIMULTANEOUS', "Simultaneous", "", pcoll["SIMULTANEOUS"].icon_id, 1),
    ]

def clear_existing_curves(obj_name):
    """仅删除与指定物体相关的曲线对象，并清理数据块"""
    curve_base_names = ["AnimationBezierCurve_", "AnimationPath_"]
    for base_name in curve_base_names:
        curve_pattern = re.compile(f"^{re.escape(base_name + obj_name)}$")
        curve_objects = [curve for curve in bpy.data.objects if curve_pattern.match(curve.name)]
        for curve in curve_objects:
            curve_data_name = curve.data.name
            bpy.data.objects.remove(curve, do_unlink=True)
            if curve_data_name in bpy.data.curves:
                curve_data = bpy.data.curves[curve_data_name]
                if curve_data.users == 0:
                    bpy.data.curves.remove(curve_data)



class VIEW3D_PT_follow_path_settings(bpy.types.Panel):
    """Creates a Panel to display and adjust Follow Path constraint settings for selected objects"""
    bl_label = "Follow Path Settings"
    bl_idname = "VIEW3D_PT_follow_path_settings"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Lazy Animation'

    @classmethod
    def poll(cls, context):
        # 只有在選擇了物體並且至少有一個物體有Follow Path約束時，才顯示面板
        return context.selected_objects and any(obj.constraints.get("MyFollowPath") for obj in context.selected_objects)

    def draw(self, context):
        layout = self.layout
        selected_objects = context.selected_objects

        # 獲取第一個具有MyFollowPath約束的物體，並以它的設置為基準顯示
        follow_path = None
        for obj in selected_objects:
            follow_path = obj.constraints.get("MyFollowPath")
            if follow_path:
                break
        
        if follow_path:
            box = layout.box()
            box.prop(follow_path, "offset_factor", text="Offset Factor")
            box.prop(follow_path, "use_fixed_location", text="Fixed Location")
            box.prop(follow_path, "use_curve_follow", text="Curve Follow")

            # 使用Operator將調整應用到所有選定物體
            layout.operator("object.apply_follow_path_settings", text="Apply Settings to All")

class OBJECT_OT_apply_follow_path_settings(bpy.types.Operator):
    """Apply Follow Path constraint settings to all selected objects"""
    bl_idname = "object.apply_follow_path_settings"
    bl_label = "Apply Follow Path Settings"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        selected_objects = context.selected_objects
        main_obj = selected_objects[0]  # 使用第一個物體作為基準
        main_constraint = main_obj.constraints.get("MyFollowPath")

        if main_constraint:
            for obj in selected_objects:
                follow_path = obj.constraints.get("MyFollowPath")
                if follow_path:
                    follow_path.offset_factor = main_constraint.offset_factor
                    follow_path.use_fixed_location = main_constraint.use_fixed_location
                    follow_path.use_curve_follow = main_constraint.use_curve_follow

            self.report({'INFO'}, "Applied settings to all selected objects.")
        else:
            self.report({'WARNING'}, "No valid Follow Path constraint found.")

        return {'FINISHED'}



class OBJECT_OT_animate_along_path(bpy.types.Operator):
    """Animate Selected Objects Along a Custom Curve Path to Cursor Position Over a Specified Number of Frames"""
    bl_idname = "object.animate_along_path"
    bl_label = "Lazy Animation"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        global initial_state

        cursor_location = context.scene.cursor.location.copy()
        frame_count = context.scene.cursor_animation_frames
        curve_type = context.scene.cursor_animation_curve_type
        key_mode = context.scene.cursor_animation_key_mode
        reverse_mode = context.scene.cursor_animation_reverse_mode
        selected_objects = context.selected_objects

        if selected_objects:
            for obj in selected_objects:
                initial_state[obj.name] = {
                    "location": obj.location.copy(),
                    "rotation": obj.rotation_euler.copy()
                }
                clear_existing_curves(obj.name)

            if key_mode == 'SIMULTANEOUS':
                start_frame = context.scene.frame_current

            for i, obj in enumerate(selected_objects):
                if curve_type == 'BEZIER':
                    curve_data = bpy.data.curves.new(name="AnimationBezierCurve_" + obj.name, type='CURVE')
                    curve_data.dimensions = '3D'
                    spline = curve_data.splines.new('BEZIER')
                    spline.bezier_points.add(1)

                    if reverse_mode:
                        spline.bezier_points[0].co = cursor_location
                        spline.bezier_points[1].co = obj.location
                    else:
                        spline.bezier_points[0].co = obj.location
                        spline.bezier_points[1].co = cursor_location

                    for point in spline.bezier_points:
                        point.handle_left_type = 'AUTO'
                        point.handle_right_type = 'AUTO'
                else:
                    curve_data = bpy.data.curves.new(name="AnimationPath_" + obj.name, type='CURVE')
                    curve_data.dimensions = '3D'
                    spline = curve_data.splines.new('POLY')
                    spline.points.add(1)

                    if reverse_mode:
                        spline.points[0].co = cursor_location.x, cursor_location.y, cursor_location.z, 1
                        spline.points[1].co = obj.location.x, obj.location.y, obj.location.z, 1
                    else:
                        spline.points[0].co = obj.location.x, obj.location.y, obj.location.z, 1
                        spline.points[1].co = cursor_location.x, cursor_location.y, cursor_location.z, 1

                curve_object = bpy.data.objects.new(curve_data.name, curve_data)
                context.collection.objects.link(curve_object)

                constraint = obj.constraints.new(type='FOLLOW_PATH')
                constraint.name = "MyFollowPath"  # 設置Follow Path的名稱
                constraint.target = curve_object
                constraint.use_fixed_location = True
                constraint.use_curve_follow = True

                if key_mode == 'SEQUENTIAL':
                    start_frame = context.scene.frame_current

                bpy.context.scene.frame_set(start_frame)
                constraint.offset_factor = 0.0
                obj.keyframe_insert(data_path="constraints[\"{}\"].offset_factor".format(constraint.name))

                bpy.context.scene.frame_set(start_frame + frame_count)
                constraint.offset_factor = 1.0
                obj.keyframe_insert(data_path="constraints[\"{}\"].offset_factor".format(constraint.name))

                obj.location = (0, 0, 0)
                obj.rotation_euler = (0, 0, 0)

            self.report({'INFO'}, "Animated {} objects along {} in {} mode over {} frames and reset to origin.".format(
                len(selected_objects), curve_type, key_mode, frame_count))
        else:
            self.report({'WARNING'}, "No objects selected")

        return {'FINISHED'}
    
    
class OBJECT_OT_cancel_animation(bpy.types.Operator):
    """Cancel the Animation and Reset Selected Objects to Original State"""
    bl_idname = "object.cancel_animation"
    bl_label = "Cancel Animation"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        global initial_state

        selected_objects = context.selected_objects

        if selected_objects:
            for obj in selected_objects:
                try:
                    if not re.search(r'\.\d+$', obj.name):
                        related_objects = [o for o in bpy.context.scene.objects if o.name.startswith(obj.name + ".") and o.select_get()]
                        if not related_objects:
                            related_objects = []

                    if obj.name in initial_state:
                        obj.location = initial_state[obj.name]["location"]
                        obj.rotation_euler = initial_state[obj.name]["rotation"]

                    obj.animation_data_clear()

                    for constraint in obj.constraints:
                        if constraint.type == 'FOLLOW_PATH':
                            obj.constraints.remove(constraint)

                    clear_existing_curves(obj.name)
                
                except ReferenceError:
                    pass

            self.report({'INFO'}, "Cancelled animation and reset {} objects to original state.".format(len(selected_objects)))
        else:
            self.report({'WARNING'}, "No objects selected")

        return {'FINISHED'}

class MEIU_PT_InfoPanel(bpy.types.Panel):
    bl_label = "Lazy Animation"
    bl_idname = "meiu_pt_blender_lazy_animation_infopanel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Lazy Animation"

    def draw_header(self, context):
        layout = self.layout
        icon = meiu_icon_collections["meiu_icon"]
        layout.label(text="", icon_value=icon.icon_id)

    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.operator("lazy_animation_meiu.button1_operator", icon_value=meiu_icon_collections["discord_icon"].icon_id)
        row.operator("lazy_animation_meiu.button2_operator", icon_value=meiu_icon_collections["twitterx_icon"].icon_id)
        row.operator("lazy_animation_meiu.button3_operator", icon_value=meiu_icon_collections["Website_icon"].icon_id)

        box = layout.box()

        col = box.column(align=True)
        col.scale_y = 1.2  # 垂直縮放，讓按鈕變得高一點
        col.operator("lazy_animation_meiu.button4_operator", icon='HELP')




class MeiUButton1Operator(bpy.types.Operator):
    bl_idname = "lazy_animation_meiu.button1_operator"
    bl_label = "Discord"

    def execute(self, context):
        webbrowser.open("https://discord.gg/jU7Mj5F3bk")
        return {'FINISHED'}

class MeiUButton2Operator(bpy.types.Operator):
    bl_idname = "lazy_animation_meiu.button2_operator"
    bl_label = "Twitter X"

    def execute(self, context):
        webbrowser.open("https://twitter.com/MeiU800")
        return {'FINISHED'}

class MeiUButton3Operator(bpy.types.Operator):
    bl_idname = "lazy_animation_meiu.button3_operator"
    bl_label = "Website"

    def execute(self, context):
        webbrowser.open("https://meiu.art/")
        return {'FINISHED'}
    
class MeiUButton4Operator(bpy.types.Operator):
    bl_idname = "lazy_animation_meiu.button4_operator"
    bl_label = "Documentation"

    def execute(self, context):
        webbrowser.open("https://syeon800.notion.site/Blender-Addons-User-Manual-034abbea8ab848b38337804e88e805c2?pvs=4")
        return {'FINISHED'}

class VIEW3D_PT_animate_along_path(bpy.types.Panel):
    """Creates a Panel in the Object properties window"""
    bl_label = "Lazy Animation"
    bl_idname = "VIEW3D_PT_animate_along_path"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Lazy Animation'

    def draw(self, context):
        layout = self.layout
        pcoll = preview_collections["main"]

        # 曲线类型选择（文字下拉式選單）
        layout.prop(context.scene, "cursor_animation_curve_type", text="Curve Type")
        layout.template_icon_view(context.scene, "cursor_animation_curve_type", show_labels=True, scale=6.0)

        # 关键帧模式选择
        layout.prop(context.scene, "cursor_animation_key_mode", text="Key Mode")
        layout.template_icon_view(context.scene, "cursor_animation_key_mode", show_labels=True, scale=6.0)

        row = layout.row()
        
        # 帧数调整滑杆
        row.prop(context.scene, "cursor_animation_frames", text="Frame Count")
        
        # 反转模式选择
        row.prop(context.scene, "cursor_animation_reverse_mode", text="Reverse Mode")

        # 添加按钮
        layout.operator("object.animate_along_path", text="Add")
        
        # 添加取消按钮
        layout.operator("object.cancel_animation", text="Remove")

classes = [
    MEIU_PT_InfoPanel,
    MeiUButton1Operator,
    MeiUButton2Operator,
    MeiUButton3Operator,
    MeiUButton4Operator,
    OBJECT_OT_animate_along_path,
    OBJECT_OT_cancel_animation,
    VIEW3D_PT_animate_along_path,
    VIEW3D_PT_follow_path_settings,
    OBJECT_OT_apply_follow_path_settings,
    
]

def register():
    # 加载缩略图和图标
    load_previews()
    load_meiu_icons()

    for cls in classes:
        bpy.utils.register_class(cls)
    
    bpy.types.Scene.cursor_animation_curve_type = bpy.props.EnumProperty(
        name="Curve Type",
        description="Type of curve to use for animation",
        items=get_curve_items
    )

    bpy.types.Scene.cursor_animation_key_mode = bpy.props.EnumProperty(
        name="Key Mode",
        description="Mode of keyframe insertion",
        items=get_keymode_items
    )

    bpy.types.Scene.cursor_animation_frames = bpy.props.IntProperty(
        name="Animation Frames",
        description="Number of frames over which the animation occurs",
        default=6,
        min=1
    )

    bpy.types.Scene.cursor_animation_reverse_mode = bpy.props.BoolProperty(
        name="Reverse Mode",
        description="Toggle to reverse the animation direction",
        default=False
    )
    translate.register()

def unregister():
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)

    if hasattr(bpy.types.Scene, "cursor_animation_curve_type"):
        del bpy.types.Scene.cursor_animation_curve_type
    if hasattr(bpy.types.Scene, "cursor_animation_key_mode"):
        del bpy.types.Scene.cursor_animation_key_mode
    if hasattr(bpy.types.Scene, "cursor_animation_frames"):
        del bpy.types.Scene.cursor_animation_frames
    if hasattr(bpy.types.Scene, "cursor_animation_reverse_mode"):
        del bpy.types.Scene.cursor_animation_reverse_mode
    translate.unregister()
    # 清理缩略图和图标
    clear_previews()

if __name__ == "__main__":
    register()
